import json 
import click
import yaml as yml

@click.command()
@click.option('--select', type=str)
def dbdoc_gen(select):
    
    with open(r"target/catalog.json") as catalog_file:
        catalog_data = json.load(catalog_file)

    with open(r"target/manifest.json") as manifest_file:
        manifest_data = json.load(manifest_file)
        
    with open(r"dbt_project.yml") as project_file:
        dbtproject = yml.safe_load(project_file)

    if type(select) != type(None):
        if '\\' in manifest_data['nodes'][list(manifest_data['nodes'].keys())[0]]['original_file_path']:
            select = select.replace('/','\\')
        else:
            select = select.replace('\\','/')
    else:
        pass

    tables = {}
    dbdiagram_string = ""
    keys_dict = {}

    for node,data in manifest_data['nodes'].items():
        if 'unrendered_config' in data.keys():
            for keys, value in data['unrendered_config'].items():
                node = node.split('.')[-1]
                if 'unique_key' in keys:
                    keys_dict[node] = value.strip()

    for model, data in catalog_data['nodes'].items():
        if ('fct' in model or 'dim' in model) and dbtproject['name'] == model.split('.')[1]:
                columns_datatypes = {}
                for col_name, col_metadata in data['columns'].items():
                    columns_datatypes[col_name] = col_metadata['type']
                tables[model] = columns_datatypes

    with open("dbdiagram_string.txt", "w") as f:

        for table, cols in tables.items():
            if dbtproject['name'] == table.split('.')[1]:
                dbdiagram_string = dbdiagram_string + "Table " + f'"{table}"' + " {\n"
                for col, dtype in cols.items():
                    if col.lower() in keys_dict.values() and col.lower().split('_')[-1] == 'skey':
                        dbdiagram_string = dbdiagram_string + f'"{col.lower()}"' + " " + f'"{dtype.lower()}"' + " " + "[pk]" "\n"
                    else:
                        dbdiagram_string = dbdiagram_string + f'"{col.lower()}"' + " " + f'"{dtype.lower()}"' + "\n"
            dbdiagram_string = dbdiagram_string + "}\n"

        print(dbdiagram_string, file=f)

        duplicates = []
        for from_table, from_cols in tables.items():
            for from_col in from_cols.keys():
                for to_table, to_cols in tables.items():
                    if from_table != to_table:
                        for to_col in to_cols.keys():
                            if to_col == from_col and (('id' in to_col.lower() or 'id' in from_col.lower()) or ('skey' in to_col.lower() or 'skey' in from_col.lower())):
                                if ''.join([from_table,from_col,to_table,to_col]) not in duplicates:
                                    print(f"Ref: "f'"{from_table.lower()}"'"."f'"{from_col.lower()}"' " - " f'"{to_table.lower()}"'"."f'"{to_col.lower()}"', file=f)
                                    duplicates.extend([''.join([from_table,from_col,to_table,to_col]),''.join([to_table,to_col,from_table,from_col])])

    f.close()
    
if __name__ == '__main__':
    dbdoc_gen()


import json
import click

from dbtgenlib import genyml

def get_manifest_data():
    with open('target/manifest.json') as manifest_file:
        manifest_data = json.load(manifest_file)
    return manifest_data

def find_keys():
    manifest_data = get_manifest_data()
    keys_dict = {}
    for node,data in manifest_data['nodes'].items():
        if 'unrendered_config' in data.keys():
            for keys, value in data['unrendered_config'].items():
                node = node.split('.')[-1]
                if 'unique_key' in keys:
                    keys_dict[node] = value.strip()
    return keys_dict

@click.command()
def check_skeys():
    model_keys = genyml.dbtdocgen()
    skeys = find_keys()
    non_unique_keys = {}

    flag = True

    for model_key in model_keys:
        for model_name, skey in skeys.items():
            if skey.strip() == model_key.strip():
                pass
            else:
                non_unique_keys[model_name] = skey.strip()
                flag = False

    if flag == False:
        # print("The test has failed!")
        click.echo("The test has failed!")
        for model_name, skey in non_unique_keys.items():
            # print(f"For the model: {model_name}, the key: {skey} is not unique!")
            click.echo(f"For the model: {model_name}, the key: {skey} is not unique!")
    else:
        click.echo("The test has been successful!")
        # print("The test has been successful!")

if __name__ == '__main__':
    check_skeys()


import click
import json
import os
import yaml as yml

@click.command()
@click.option('--select', type=str)
def dbtdocgen(select):
    
    keys_list = []

    with open(r"target/manifest.json") as file:
        manifest_data = json.load(file)
        
    with open(r"dbt_project.yml") as file:
        dbtproject = yml.safe_load(file)
 
    names_set = set()

    if type(select) != type(None):
        if '\\' in manifest_data['nodes'][list(manifest_data['nodes'].keys())[0]]['original_file_path']:
            select = select.replace('/','\\')
        else:
            select = select.replace('\\','/')
    else:
        pass

    for node, data in manifest_data['nodes'].items():
        if 'unique_key' in data['unrendered_config']:
            names = str(data['fqn'][-2])
            names_set.add(names)

    distinct_names = list(names_set)

    for name in distinct_names:
        yaml = '\nversion: 2\n\nmodels:\n\n'

        for node, data in sorted(list(manifest_data['nodes'].items()), key=lambda x: x[1]['fqn'][-1]):
            if str(select) in data['original_file_path']:
                if name in data['fqn'][-2]:
                    if 'unique_key' in data['unrendered_config'] and len(data['fqn']) != 0 and dbtproject['name'] == data['fqn'][0]:
                        yaml += f"- name: {data['fqn'][-1]}\n"
                        yaml += "  description: This is a table in staging\n"
                        yaml += "  columns:\n"
                        yaml += f"   - name: {data['unrendered_config']['unique_key']}\n"
                        yaml += "     description: This is a surrogate key\n"
                        yaml += "     tests:\n"
                        yaml += f"      - not_null\n"
                        yaml += f"      - unique\n"
                        yaml += f"      \n"
                        file_name = f"_{name}_doc.yml"  # Use the name as the file name
                        file_path = os.path.join("\\".join(data['original_file_path'].split('\\')[:-1]), file_name)  # Specify the output folder path

                    else:
                        continue
                
                    with open(file_path, 'w') as file:
                        file.write(yaml)

                    keys_list.append(data['unrendered_config']['unique_key'].strip())

    return keys_list

if __name__ == '__main__':
    dbtdocgen()


from setuptools import setup, find_packages
setup(
    name='dbtgenlib',
    version='0.0.38',
    packages=find_packages(),
    install_requires=[
        'click',
    ],
    entry_points='''
        [console_scripts]
        yml_gen=dbtgenlib.genyml:dbtdocgen
        doc_gen=dbtgenlib.gendoc:dbdoc_gen
        key_test=dbtgenlib.testkeys:check_skeys
        ''',
)